# SAP BusinessObjects BI Platform Cookbook

SAP BusinessObjects BI Platform cookbook (shortly bo-bip) installs SAP BO BIP. Later it will support configuration of BIP services & servers.

## Scope

Currently bo-bip cookbook is a POC supporting only a few versions of BIP and deployment patterns. For example, only SQL Anywhere is supported as backend database.

If you are new to chef, and you want just create a VM from ready-to-use installation SAP BO BIP files, use [QUICK-STEPS.md](doc-files/QUICK-STEPS.md).

## Requirements

 - Installation files for BIP available through an HTTP url
 - Chef 14.8 or later

## Platform Support

 - RHEL 7.5
 - Windows 2012 R2

Platforms supported by SAP PAM (see [KB 1338845](https://launchpad.support.sap.com/#/notes/1338845)) can be added.

## Attributes

 - `node['bo-bip']['group']` - System group for ownership. Default to 'bo-bip'. This attribute is not used on Windows platforms.
 - `node['bo-bip']['owner']` - System user for ownership. Default to 'bo-bip'. This attribute is not used on Windows platforms.
 - `node['bo-bip']['base_uri']` - Base URI to the directory of initial installation files of BIP. A file `checksum.sha` is required in this directory (see [Checksum File](#checksum-file) below). Default to `nil`. Set a value is **required**.
 - `node['bo-bip']['base_uri_upd']` - (Optional) Base URI to the directory of update files of BIP. A file `checksum.sha` is required in this directory (see [Checksum File](#checksum-file) below). Default to `nil` (no update to install).
 - `node['bo-bip']['base_uri_hotfix']` - (Optional) Base URI to the directory of hotfix files of BIP. A file `checksum.sha` is required in this directory (see [Checksum File](#checksum-file) below). Default to `nil` (no hotfix to install).
 - `node['bo-bip']['install_path']` - Installation path. Default to '/home/bo-bip/SapBusinessObjectsBIP'.
 - `node['bo-bip']['password']` - Password for BIP Administrator user. The same value is also used for BIP cluster key and SQL Anywhere admin user.
 - `node['bo-bip']['product_key']` - BIP product key valid at the time of installation.

## Usage

Add `bo-bip::default` to your runlist.

## Supported SAP BO BIP Versions

![Table Versions](doc-files/Table Versions.png)

## Checksum file

The three parameters `base_uri`, `base_uri_upd`, `base_uri_hotfix` must reference a directory containing `checksum.sha` file. It's an entry point for the cookbook to download, and verify checksum of BIP installation files.

Here is an example of `checksum.sha` file :

```
42f924ca2248c1d93704d8dcccc7d9eedf4cc6ba712f322b6d35ead9ca4d6135  51052539_part1.exe
26d3f7c77b116355e43a1e64782bc9b25ad34b682594edeb7fdff9fa99602d99  51052539_part2.rar
```

Here is example commands to write the file :

```
mkdir -p files/42_sp05_lin
cd files/42_sp05_lin
mv ~/Downloads/51052539_part1.exe .
mv ~/Downloads/51052539_part2.rar .
shasum -a 256 *.exe *.rar > checksum.sha
```

## Contribution

Pull requests are welcome as long as following commands are executed successfully:

```
chef exec rake
kitchen test
```

**Warning**: Tests executed by Kitchen requires more than 10 hours to be completed.

## Author

**Author**: Jean-Pierre Matsumoto ([jpmat296@gmail.com](mailto:jpmat296@gmail.com))
