#
# Cookbook:: bo-bip
# Recipe:: default
#
# Copyright:: 2018, The Authors, All Rights Reserved.

linux_environment = {
  'LC_ALL' => 'en_US.UTF-8',
}

if platform?('redhat')

  package 'bip-deps-server-rpms' do
    package_name [
      'libstdc++.x86_64', 'libstdc++.i686',
      'glibc.i686', 'glibc.x86_64',
      'libX11.i686', 'libX11.x86_64',
      'libXext.i686', 'libXext.x86_64',
      'expat.i686', 'expat.x86_64',
      'libgcc.i686', 'libgcc.x86_64',
      'libXcursor.i686', 'libXcursor.x86_64',
      'libXrender.i686', 'libXrender.x86_64',
      'libXfixes.i686', 'libXfixes.x86_64',
      'libxcb.i686', 'libxcb.x86_64',
      'libXau.i686', 'libXau.x86_64'
    ]
    action :upgrade
  end

  package 'bip-deps-server-optional-rpms' do
    package_name %w(compat-libstdc++-33.i686 compat-libstdc++-33.x86_64)
    action :upgrade
  end

  group node['bo-bip']['group']

  user node['bo-bip']['owner'] do
    comment 'User for SAP BusinessObjects Business Intelligence platform'
    group node['bo-bip']['group']
    home '/home/bo-bip'
    manage_home true
  end

end

if platform?('redhat')

  directory "#{Chef::Config['file_cache_path']}/install_files" do
    owner node['bo-bip']['owner']
    group node['bo-bip']['group']
    mode '0755'
  end

elsif platform?('windows')

  reboot 'initial reboot' do
    action :reboot_now
    only_if { reboot_pending? }
  end

  directory "#{Chef::Config['file_cache_path']}/install_files"

end

cs = remote_file "#{Chef::Config['file_cache_path']}/checksum.sha" do
  source "#{node['bo-bip']['base_uri']}/checksum.sha"
end
cs.run_action(:create)

filelist = []
open("#{Chef::Config['file_cache_path']}/checksum.sha").each do |line|
  splits = line.split
  filelist << { 'filename' => splits[1], 'checksum' => splits[0] }
end

filelist.each do |file|
  if platform?('redhat')
    remote_file "#{Chef::Config['file_cache_path']}/install_files/#{file['filename']}" do
      source "#{node['bo-bip']['base_uri']}#{file['filename']}"
      checksum file['checksum']
      owner node['bo-bip']['owner']
      group node['bo-bip']['group']
      not_if { File.exist?("#{node['bo-bip']['install_path']}/InstallData/inventory.txt") }
    end
  elsif platform?('windows')
    remote_file "#{Chef::Config['file_cache_path']}/install_files/#{file['filename']}" do
      source "#{node['bo-bip']['base_uri']}#{file['filename']}"
      checksum file['checksum']
      not_if { File.exist?("#{node['bo-bip']['install_path']}/InstallData/inventory.txt") }
    end
  end
end

setup_dir = ''

if platform?('redhat')

  remote_file "#{Chef::Config['file_cache_path']}/unrar-5.4.0-1.el6.x86_64.rpm" do
    source 'https://forensics.cert.org/centos/cert/6/x86_64/unrar-5.4.0-1.el6.x86_64.rpm'
    owner 'root'
    group 'root'
    mode '0644'
  end

  rpm_package 'unrar' do
    source "#{Chef::Config['file_cache_path']}/unrar-5.4.0-1.el6.x86_64.rpm"
  end

  if one_install?(filelist[0]['filename'])

    execute 'unrar_files' do
      cwd "#{Chef::Config['file_cache_path']}/install_files"
      command "unrar x #{filelist[0]['filename']}"
      user node['bo-bip']['owner']
      group node['bo-bip']['group']
      environment linux_environment
      creates 'BISERVONE.tgz'
      not_if { File.exist?("#{node['bo-bip']['install_path']}/InstallData/inventory.txt") }
    end

    directory "#{Chef::Config['file_cache_path']}/install_files/BISERVONE" do
      owner node['bo-bip']['owner']
      group node['bo-bip']['group']
      mode '0700'
      not_if { File.exist?("#{node['bo-bip']['install_path']}/InstallData/inventory.txt") }
    end

    execute 'untar_files' do
      cwd "#{Chef::Config['file_cache_path']}/install_files"
      command 'tar xzf BISERVONE.tgz -C BISERVONE'
      user node['bo-bip']['owner']
      group node['bo-bip']['group']
      environment linux_environment
      creates 'BISERVONE/ProductId.txt'
      not_if { File.exist?("#{node['bo-bip']['install_path']}/InstallData/inventory.txt") }
    end

    setup_dir = "#{Chef::Config['file_cache_path']}/install_files/BISERVONE"

  else

    execute 'unrar_files' do
      cwd "#{Chef::Config['file_cache_path']}/install_files"
      command "unrar x #{filelist[0]['filename']}"
      user node['bo-bip']['owner']
      group node['bo-bip']['group']
      environment linux_environment
      creates filelist[0]['filename'].split('_')[0].to_s
      not_if { File.exist?("#{node['bo-bip']['install_path']}/InstallData/inventory.txt") }
    end

    setup_dir = "#{Chef::Config['file_cache_path']}/install_files/#{filelist[0]['filename'].split('_')[0]}/DATA_UNITS/BusinessObjectsServer_lnx"

  end

elsif platform?('windows')

  include_recipe 'seven_zip'

  if one_install?(filelist[0]['filename'])

    directory "#{Chef::Config['file_cache_path']}/install_files/one" do
      not_if { File.exist?("#{node['bo-bip']['install_path']}/InstallData/inventory.txt") }
    end

    seven_zip_archive 'unrar_files' do
      source "#{Chef::Config['file_cache_path']}/install_files/#{filelist[0]['filename']}"
      checksum filelist[0]['checksum'].to_s
      path "#{Chef::Config['file_cache_path']}/install_files/one"
      timeout 20 * 60
      not_if { File.exist?("#{Chef::Config['file_cache_path']}/install_files/one/setup.exe") }
      not_if { File.exist?("#{node['bo-bip']['install_path']}/InstallData/inventory.txt") }
    end

    setup_dir = "#{Chef::Config['file_cache_path']}/install_files/one"

  else

    seven_zip_archive 'unrar_files' do
      source "#{Chef::Config['file_cache_path']}/install_files/#{filelist[0]['filename']}"
      checksum filelist[0]['checksum'].to_s
      path "#{Chef::Config['file_cache_path']}/install_files"
      timeout 20 * 60
      not_if { File.exist?("#{Chef::Config['file_cache_path']}/install_files/#{filelist[0]['filename'].split('_')[0]}") }
      not_if { File.exist?("#{node['bo-bip']['install_path']}/InstallData/inventory.txt") }
    end

    setup_dir = "#{Chef::Config['file_cache_path']}/install_files/#{filelist[0]['filename'].split('_')[0]}/DATA_UNITS/BusinessObjectsServer_win"

  end

end

if platform?('redhat')

  directory node['bo-bip']['install_path'] do
    owner node['bo-bip']['owner']
    group node['bo-bip']['group']
    mode '0755'
    recursive true
  end

  template "#{node['bo-bip']['install_path']}/response.ini" do
    source "#{bip_install_version_string(filelist[0]['filename'])}/response.ini.erb"
    owner node['bo-bip']['owner']
    group node['bo-bip']['group']
    mode '0600'
    action :nothing
    subscribes :create, 'execute[setup.sh response.ini]', :before
  end

elsif platform?('windows')

  directory node['bo-bip']['install_path']

  template "#{node['bo-bip']['install_path']}/response.ini" do
    source "#{bip_install_version_string(filelist[0]['filename'])}/response.ini.erb"
    action :nothing
    subscribes :create, 'execute[setup.exe response.ini]', :before
  end

end

log "SAP BO BIP Installation detected with version string '#{bip_install_version_string(filelist[0]['filename'])}'"

if platform?('redhat')

  command =
    if one_install?(filelist[0]['filename'])
      "./setup.sh -InstallDir #{node['bo-bip']['install_path']} -r #{node['bo-bip']['install_path']}/response.ini"
    else
      "./setup.sh -r #{node['bo-bip']['install_path']}/response.ini"
    end

  execute 'setup.sh response.ini' do
    cwd setup_dir
    command command
    user node['bo-bip']['owner']
    group node['bo-bip']['group']
    environment linux_environment
    timeout 3 * 60 * 60
    creates "#{node['bo-bip']['install_path']}/InstallData/inventory.txt"
  end

  file "#{node['bo-bip']['install_path']}/response.ini" do
    backup false
    action :delete
  end

  filelist.each do |file|
    file "#{Chef::Config['file_cache_path']}/install_files/#{file['filename']}" do
      backup false
      action :delete
    end
  end

  file "#{Chef::Config['file_cache_path']}/install_files/BISERVONE.tgz}" do
    backup false
    action :delete
  end

  directory "#{Chef::Config['file_cache_path']}/install_files/BISERVONE" do
    action :delete
    recursive true
  end

  directory "#{Chef::Config['file_cache_path']}/install_files/#{filelist[0]['filename'].split('_')[0]}" do
    action :delete
    recursive true
  end

  execute 'ccm.sh -managedstop WACS' do
    cwd "#{node['bo-bip']['install_path']}/sap_bobj"
    command "./ccm.sh -managedstop Node1.WebApplicationContainerServer -username Administrator -password #{node['bo-bip']['password']}"
    user node['bo-bip']['owner']
    group node['bo-bip']['group']
    environment linux_environment
    action :nothing
    subscribes :run, 'execute[setup.sh response.ini]', :immediately
  end

  execute 'setupinit.sh' do
    command "#{node['bo-bip']['install_path']}/sap_bobj/init/setupinit.sh"
    user 'root'
    group 'root'
    environment linux_environment
    creates '/etc/init.d/SAPBOBJEnterpriseXI40'
  end

elsif platform?('windows')

  execute 'setup.exe response.ini' do
    cwd setup_dir
    command "setup.exe -r #{node['bo-bip']['install_path']}/response.ini"
    timeout 3 * 60 * 60
    creates "#{node['bo-bip']['install_path']}/InstallData/inventory.txt"
  end

  file "#{node['bo-bip']['install_path']}/response.ini" do
    backup false
    action :delete
  end

  filelist.each do |file|
    file "#{Chef::Config['file_cache_path']}/install_files/#{file['filename']}" do
      backup false
      action :delete
    end
  end

  directory "#{Chef::Config['file_cache_path']}/install_files/one" do
    action :delete
    recursive true
  end

  directory "#{Chef::Config['file_cache_path']}/install_files/#{filelist[0]['filename'].split('_')[0]}" do
    action :delete
    recursive true
  end

  powershell_script 'ccm.exe -managedstop WACS' do
    cwd "#{node['bo-bip']['install_path']}/SAP BusinessObjects Enterprise XI 4.0/win64_x64"
    code <<-EOH
      ./ccm.exe -managedstop Node1.WebApplicationContainerServer \
        -username Administrator \
        -password #{node['bo-bip']['password']}
    EOH
    only_if win_wacs_is_running_cmd
  end

end

unless node['bo-bip']['base_uri_upd'].nil?

  cs = remote_file "#{Chef::Config['file_cache_path']}/checksum_upd.sha" do
    source "#{node['bo-bip']['base_uri_upd']}/checksum.sha"
  end
  cs.run_action(:create)

  filelist_upd = []
  open("#{Chef::Config['file_cache_path']}/checksum_upd.sha").each do |line|
    splits = line.split
    filelist_upd << { 'filename' => splits[1], 'checksum' => splits[0] }
  end

  filelist_upd.each do |file|
    if platform?('redhat')
      remote_file "#{Chef::Config['file_cache_path']}/install_files/#{file['filename']}" do
        source "#{node['bo-bip']['base_uri_upd']}#{file['filename']}"
        checksum file['checksum']
        owner node['bo-bip']['owner']
        group node['bo-bip']['group']
        not_if { inventory_contains?('update') }
      end
    elsif platform?('windows')
      remote_file "#{Chef::Config['file_cache_path']}/install_files/#{file['filename']}" do
        source "#{node['bo-bip']['base_uri_upd']}#{file['filename']}"
        checksum file['checksum']
        not_if { inventory_contains?('update') }
      end
    end
  end

  if platform?('redhat')

    directory "#{Chef::Config['file_cache_path']}/install_files/BIPLATFORM" do
      owner node['bo-bip']['owner']
      group node['bo-bip']['group']
      mode '0700'
      not_if { inventory_contains?('update') }
    end

    execute 'unrar_upd_files' do
      cwd "#{Chef::Config['file_cache_path']}/install_files"
      command "unrar x #{filelist_upd[0]['filename']}"
      user node['bo-bip']['owner']
      group node['bo-bip']['group']
      environment linux_environment
      creates 'BIPLATFORM.tgz'
      not_if { inventory_contains?('update') }
    end

    execute 'untar_upd_files' do
      cwd "#{Chef::Config['file_cache_path']}/install_files"
      command 'tar xzf BIPLATFORM.tgz -C BIPLATFORM'
      user node['bo-bip']['owner']
      group node['bo-bip']['group']
      environment linux_environment
      creates 'BIPLATFORM/ProductId.txt'
      not_if { inventory_contains?('update') }
    end

  elsif platform?('windows')

    directory "#{Chef::Config['file_cache_path']}/install_files/BIPLATFORM" do
      not_if { inventory_contains?('update') }
    end

    seven_zip_archive 'unrar_upd_files' do
      source "#{Chef::Config['file_cache_path']}/install_files/#{filelist_upd[0]['filename']}"
      checksum filelist_upd[0]['filename'].to_s
      path "#{Chef::Config['file_cache_path']}/install_files/BIPLATFORM"
      timeout 20 * 60
      not_if { File.exist?("#{Chef::Config['file_cache_path']}/install_files/BIPLATFORM/setup.exe") }
      not_if { inventory_contains?('update') }
    end

  end

  if platform?('redhat')

    template "#{node['bo-bip']['install_path']}/upd_response.ini" do
      source "#{bip_update_version_string(filelist_upd[0]['filename'])}/response.ini.erb"
      owner node['bo-bip']['owner']
      group node['bo-bip']['group']
      mode '0600'
      action :nothing
      subscribes :create, 'execute[setup.sh upd_response.ini]', :before
    end

  elsif platform?('windows')

    template "#{node['bo-bip']['install_path']}/upd_response.ini" do
      source "#{bip_update_version_string(filelist_upd[0]['filename'])}/response.ini.erb"
      action :nothing
      subscribes :create, 'execute[setup.exe upd_response.ini]', :before
    end

  end

  log "SAP BO BIP Update detected with version string '#{bip_update_version_string(filelist_upd[0]['filename'])}'"

  if platform?('redhat')

    execute 'setup.sh upd_response.ini' do
      cwd "#{Chef::Config['file_cache_path']}/install_files/BIPLATFORM"
      command "./setup.sh -r #{node['bo-bip']['install_path']}/upd_response.ini"
      user node['bo-bip']['owner']
      group node['bo-bip']['group']
      environment linux_environment
      timeout 3 * 60 * 60
      not_if { inventory_contains?('update') }
    end

    file "#{node['bo-bip']['install_path']}/upd_response.ini" do
      backup false
      action :delete
    end

    filelist_upd.each do |file|
      file "#{Chef::Config['file_cache_path']}/install_files/#{file['filename']}" do
        backup false
        action :delete
      end
    end

    file "#{Chef::Config['file_cache_path']}/install_files/BIPLATFORM.tgz" do
      backup false
      action :delete
    end

    directory "#{Chef::Config['file_cache_path']}/install_files/BIPLATFORM" do
      action :delete
      recursive true
    end

    execute 'ccm.sh -managedstop WACS' do
      cwd "#{node['bo-bip']['install_path']}/sap_bobj"
      command "./ccm.sh -managedstop Node1.WebApplicationContainerServer -username Administrator -password #{node['bo-bip']['password']}"
      user node['bo-bip']['owner']
      group node['bo-bip']['group']
      environment linux_environment
      action :nothing
      subscribes :run, 'execute[setup.sh upd_response.ini]', :immediately
    end

  elsif platform?('windows')

    execute 'setup.exe upd_response.ini' do
      cwd "#{Chef::Config['file_cache_path']}/install_files/BIPLATFORM"
      command "setup.exe -r #{node['bo-bip']['install_path']}/upd_response.ini"
      timeout 3 * 60 * 60
      not_if { inventory_contains?('update') }
    end

    file "#{node['bo-bip']['install_path']}/upd_response.ini" do
      backup false
      action :delete
    end

    filelist_upd.each do |file|
      file "#{Chef::Config['file_cache_path']}/install_files/#{file['filename']}" do
        backup false
        action :delete
      end
    end

    directory "#{Chef::Config['file_cache_path']}/install_files/BIPLATFORM" do
      action :delete
      recursive true
    end

    windows_service 'BOEXI40Tomcat' do
      subscribes :stop, 'execute[setup.exe upd_response.ini]', :immediately
      timeout 300
      only_if { reboot_pending? }
    end

    windows_service 'BOEXI40SIANode1' do
      subscribes :stop, 'execute[setup.exe upd_response.ini]', :immediately
      timeout 300
      only_if { reboot_pending? }
    end

    windows_service 'SQLANYs_SQLAnywhereForBI' do
      subscribes :stop, 'execute[setup.exe upd_response.ini]', :immediately
      timeout 300
      only_if { reboot_pending? }
    end

    reboot 'reboot after update' do
      subscribes :reboot_now, 'execute[setup.exe upd_response.ini]', :immediately
      only_if { reboot_pending? }
    end

    powershell_script 'ccm.exe -managedstop WACS' do
      cwd "#{node['bo-bip']['install_path']}/SAP BusinessObjects Enterprise XI 4.0/win64_x64"
      code <<-EOH
        ./ccm.exe -managedstop Node1.WebApplicationContainerServer \
          -username Administrator \
          -password #{node['bo-bip']['password']}
      EOH
      only_if win_wacs_is_running_cmd
    end

  end

end

unless node['bo-bip']['base_uri_hotfix'].nil?

  cs = remote_file "#{Chef::Config['file_cache_path']}/checksum_hotfix.sha" do
    source "#{node['bo-bip']['base_uri_hotfix']}/checksum.sha"
  end
  cs.run_action(:create)

  filelist_hf = []
  open("#{Chef::Config['file_cache_path']}/checksum_hotfix.sha").each do |line|
    splits = line.split
    filelist_hf << { 'filename' => splits[1], 'checksum' => splits[0] }
  end

  filelist_hf.each do |file|
    if platform?('redhat')
      remote_file "#{Chef::Config['file_cache_path']}/install_files/#{file['filename']}" do
        source "#{node['bo-bip']['base_uri_hotfix']}#{file['filename']}"
        checksum file['checksum']
        owner node['bo-bip']['owner']
        group node['bo-bip']['group']
        not_if { inventory_contains?('Hotfix') }
      end
    elsif platform?('windows')
      remote_file "#{Chef::Config['file_cache_path']}/install_files/#{file['filename']}" do
        source "#{node['bo-bip']['base_uri_hotfix']}#{file['filename']}"
        checksum file['checksum']
        not_if { inventory_contains?('Hotfix') }
      end
    end
  end

  if platform?('redhat')

    directory "#{Chef::Config['file_cache_path']}/install_files/hotfix" do
      owner node['bo-bip']['owner']
      group node['bo-bip']['group']
      mode '0700'
      not_if { inventory_contains?('Hotfix') }
    end

    execute 'untar_hotfix_files' do
      cwd "#{Chef::Config['file_cache_path']}/install_files"
      command "tar xzf #{filelist_hf[0]['filename']} -C hotfix"
      user node['bo-bip']['owner']
      group node['bo-bip']['group']
      environment linux_environment
      creates 'hotfix/ProductId.txt'
      not_if { inventory_contains?('Hotfix') }
    end

  elsif platform?('windows')

    directory "#{Chef::Config['file_cache_path']}/install_files/hotfix" do
      not_if { inventory_contains?('Hotfix') }
    end

    seven_zip_archive 'unrar_hotfix_files' do
      source "#{Chef::Config['file_cache_path']}/install_files/#{filelist_hf[0]['filename']}"
      checksum filelist_hf[0]['filename'].to_s
      path "#{Chef::Config['file_cache_path']}/install_files/hotfix"
      timeout 20 * 60
      not_if { File.exist?("#{Chef::Config['file_cache_path']}/install_files/hotfix/setup.exe") }
      not_if { inventory_contains?('Hotfix') }
    end

  end

  if platform?('redhat')

    template "#{node['bo-bip']['install_path']}/hf_response.ini" do
      source "#{bip_hotfix_version_string(filelist_hf[0]['filename'])}/response.ini.erb"
      owner node['bo-bip']['owner']
      group node['bo-bip']['group']
      mode '0600'
      action :nothing
      subscribes :create, 'execute[setup.sh hf_response.ini]', :before
    end

  elsif platform?('windows')

    template "#{node['bo-bip']['install_path']}/hf_response.ini" do
      source "#{bip_hotfix_version_string(filelist_hf[0]['filename'])}/response.ini.erb"
      action :nothing
      subscribes :create, 'execute[setup.exe hf_response.ini]', :before
    end

  end

  log "SAP BO BIP Hotfix detected with version string '#{bip_hotfix_version_string(filelist_hf[0]['filename'])}'"

  if platform?('redhat')

    execute 'setup.sh hf_response.ini' do
      cwd "#{Chef::Config['file_cache_path']}/install_files/hotfix"
      command "./setup.sh -r #{node['bo-bip']['install_path']}/hf_response.ini"
      user node['bo-bip']['owner']
      group node['bo-bip']['group']
      environment linux_environment
      timeout 3 * 60 * 60
      not_if { inventory_contains?('Hotfix') }
    end

    file "#{node['bo-bip']['install_path']}/hf_response.ini" do
      backup false
      action :delete
    end

    filelist_hf.each do |file|
      file "#{Chef::Config['file_cache_path']}/install_files/#{file['filename']}" do
        backup false
        action :delete
      end
    end

    directory "#{Chef::Config['file_cache_path']}/install_files/hotfix" do
      action :delete
      recursive true
    end

    execute 'ccm.sh -managedstop WACS' do
      cwd "#{node['bo-bip']['install_path']}/sap_bobj"
      command "./ccm.sh -managedstop Node1.WebApplicationContainerServer -username Administrator -password #{node['bo-bip']['password']}"
      user node['bo-bip']['owner']
      group node['bo-bip']['group']
      environment linux_environment
      action :nothing
      subscribes :run, 'execute[setup.sh hf_response.ini]', :immediately
    end

  elsif platform?('windows')

    execute 'setup.exe hf_response.ini' do
      cwd "#{Chef::Config['file_cache_path']}/install_files/hotfix"
      command "setup.exe -r #{node['bo-bip']['install_path']}/hf_response.ini"
      timeout 3 * 60 * 60
      not_if { inventory_contains?('Hotfix') }
    end

    file "#{node['bo-bip']['install_path']}/hf_response.ini" do
      backup false
      action :delete
    end

    filelist_hf.each do |file|
      file "#{Chef::Config['file_cache_path']}/install_files/#{file['filename']}" do
        backup false
        action :delete
      end
    end

    directory "#{Chef::Config['file_cache_path']}/install_files/hotfix" do
      action :delete
      recursive true
    end

    windows_service 'BOEXI40Tomcat' do
      subscribes :stop, 'execute[setup.exe hf_response.ini]', :immediately
      timeout 300
      only_if { reboot_pending? }
    end

    windows_service 'BOEXI40SIANode1' do
      subscribes :stop, 'execute[setup.exe hf_response.ini]', :immediately
      timeout 300
      only_if { reboot_pending? }
    end

    windows_service 'SQLANYs_SQLAnywhereForBI' do
      subscribes :stop, 'execute[setup.exe hf_response.ini]', :immediately
      timeout 300
      only_if { reboot_pending? }
    end

    reboot 'reboot after hotfix' do
      subscribes :reboot_now, 'execute[setup.exe upd_response.ini]', :immediately
      only_if { reboot_pending? }
    end

    powershell_script 'ccm.exe -managedstop WACS' do
      cwd "#{node['bo-bip']['install_path']}/SAP BusinessObjects Enterprise XI 4.0/win64_x64"
      code <<-EOH
        ./ccm.exe -managedstop Node1.WebApplicationContainerServer \
          -username Administrator \
          -password #{node['bo-bip']['password']}
      EOH
      only_if win_wacs_is_running_cmd
    end

  end

end

# Workaround for issue https://github.com/inspec/inspec/issues/2822
if platform?('windows')
  remote_file "#{Chef::Config['file_cache_path']}/install_files/version.xml" do
    source 'http://localhost:8080/biprws/raylight/v1/about'
  end
end
